/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup packet
 *  @{
 */

/** @file
 *  Packet map and queue.
 */

#ifndef __NET_PACKET_H__
#define __NET_PACKET_H__

/** Packet identifier type.
 *  Value zero (0) is used as an invalid identifier.
 */
typedef int	packet_id_t;

/** Type definition of the packet.
 *  @see packet
 */
typedef struct packet *	packet_t;

/** Type definition of the packet pointer.
 *  @see packet
 */
typedef packet_t *		packet_ref;

/** Type of the received packet.
 */
enum packet_type{
	/** The packet is from the local subsystem.
	 */
	PACKET_SELF,
	/** The packet is for all hosts.
	 */
	PACKET_BROADCAST,
	/** The packet target complies with the local subsystem filter.
	 */
	PACKET_MULTICAST,
	/** The packet is for the local subsystem from other host.
	 */
	PACKET_OTHERHOST
};

/** Finds the packet mapping.
 *  @param packet_id The packet identifier to be found. Input parameter.
 *  @returns The found packet reference.
 *  @returns NULL if the mapping does not exist.
 */
packet_t	pm_find( packet_id_t packet_id );

/** Adds the packet mapping.
 *  @param packet The packet to be remembered. Input parameter.
 *  @returns EOK on success.
 *  @returns EINVAL if the packet is not valid.
 *  @returns EINVAL if the packet map is not initialized.
 *  @returns ENOMEM if there is not enough memory left.
 */
int	pm_add( packet_t packet );

/** Initializes the packet map.
 *  @returns EOK on success.
 *  @returns ENOMEM if there is not enough memory left.
 */
int	pm_init( void );

/** Releases the packet map.
 */
void	pm_destroy( void );

/** Add packet to the sorted queue.
 *  The queue is sorted in the ascending order.
 *  The packet is inserted right before the packets of the same order value.
 *  @param first The first packet of the queue. May be NULL. Input parameter.
 *  @param packet The packet to be added. Input parameter.
 *  @param order The packet order value. Input parameter.
 *  @param metric The metric value of the packet. Input parameter.
 *  @returns The first packet of the queue. The original first packet may be shifted by the new packet.
 *  @returns NULL if the packet is not valid.
 */
packet_t	pq_add( packet_t first, packet_t packet, int order, size_t metric );

/** Inserts packet after the given one.
 *  @param packet The packet in the queue. Input parameter.
 *  @param new_packet The new packet to be inserted.
 *  @returns EOK on success.
 *  @returns EINVAL if etiher of the packets is invalid.
 */
int	pq_insert_after( packet_t packet, packet_t new_packet );

/** Detach the packet from the queue.
 *  @param packet The packet to be detached. Input parameter.
 *  @returns The next packet in the queue. If the packet is the first one of the queue, this becomes the new first one.
 *  @returns NULL if there is no packet left.
 *  @returns NULL if the packet is not valid.
 */
packet_t	pq_detach( packet_t packet );

/** Sets the packet order and metric attributes.
 *  @param packet The packet to be set. Input parameter.
 *  @param order The packet order value. Input parameter.
 *  @param metric The metric value of the packet. Input parameter.
 */
int	pq_set( packet_t packet, int order, size_t metric );

/** Releases the whole queue.
 *  Detaches all packets of the queue and calls the packet_release() for each of them.
 *  @param first The first packet of the queue. Input parameter.
 *  @param packet_release The releasing function called for each of the packets after its detachment. Input parameter.
 */
void	pq_destroy( packet_t first, void ( * packet_release )( packet_t packet ));

/** Returns the next packet in the queue.
 *  @param packet The packet queue member. Input parameter.
 *  @returns The next packet in the queue.
 *  @returns NULL if there is no next packet.
 *  @returns NULL if the packet is not valid.
 */
packet_t	pq_next( packet_t packet );

/** Returns the previous packet in the queue.
 *  @param packet The packet queue member. Input parameter.
 *  @returns The previous packet in the queue.
 *  @returns NULL if there is no previous packet.
 *  @returns NULL if the packet is not valid.
 */
packet_t	pq_previous( packet_t packet );

#endif

/** @}
 */
